@@ -15,6 +15,7 @@ def create_exec_file(id_list, command_template, indir, outdir, output_filename_t
             ols = ' '.join([x.replace("<id>", idx) for x in output_filename_templates])
             exec_file.write('o{0}:\t{1}\n'.format(str(ip).zfill(6), ols))
 
+
 def check_remote_path(remote_machine, root_path):
     """Checks existence of parent folder of remote root path
     and non-existence of remote root path itself"""
@@ -125,10 +126,11 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
 
     if protocol == 'remote' and not env_and_do_not_replicate:
         check_remote_path(remote_machine, fs_locations["runtime_root"])
+        """
         if env_instr:
             tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_root"], "{0}:{1}".format(remote_machine, fs_locations["runtime_root"])]
             if DEBUG:
-                print("COMMAND", tr_cmd)
+                print("COMMAND", " ".join(tr_cmd))
                 p = subprocess.Popen(tr_cmd)
             else:
                 p = subprocess.Popen(tr_cmd,
@@ -139,13 +141,13 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
             mkd2_cmd = ["ssh", remote_machine, "mkdir", fs_locations["runtime_exec"]]
             tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_shared"], "{0}:{1}".format(remote_machine, fs_locations["runtime_shared"])]
             if DEBUG:
-                print("COMMAND", mkd1_cmd)
+                print("COMMAND", " ".join(mkd1_cmd))
                 p = subprocess.Popen(mkd1_cmd)
                 p.wait()
-                print("COMMAND", mkd2_cmd)
+                print("COMMAND", " ".join(mkd2_cmd))
                 p = subprocess.Popen(mkd2_cmd)
                 p.wait()
-                print("COMMAND", tr_cmd)
+                print("COMMAND", " ".join(tr_cmd))
                 p = subprocess.Popen(tr_cmd)
             else:
                 p = subprocess.Popen(
@@ -161,16 +163,16 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                     stderr=devnull, stdout=devnull)
             p.wait()
         time.sleep(10)
+        """
 
     # Step 3: create Work sub-filesystem: "custom" or "locusts" mode -------------------
     if env_instr:  # "custom" mode: environment is created following instruction file
         # Replace tags in instruction file lines
         #  Choose replacement whether it is in remote or in local
         if protocol == "remote":
-            mkdircmd = "ssh {0} mkdir".format(remote_machine)
-            copycmd = "bash {0}".format(data_transfer_protocol)
-            runtime_envroot_cp = "{0}:{1}".format(remote_machine, fs_locations["runtime_work"])
-            runtime_envroot_mkdir = fs_locations["runtime_work"]
+            mkdircmd, copycmd = "mkdir", "cp"
+            runtime_envroot_cp = fs_locations["build_work"]
+            runtime_envroot_mkdir = fs_locations["build_work"]
             workpath = fs_locations["runtime_work"]
         elif protocol == 'remote-sharedfs':
             mkdircmd, copycmd = "mkdir", "cp"
@@ -183,20 +185,26 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
         #  Read the instruction file, replace tags and execute
         workdir, instructions = parse_fs_tree(env_instr, build_envroot)
         if protocol != 'local' and not env_and_do_not_replicate:
+            compiled_instructions = []
             for instr in instructions:
                 cmd = instr.replace("<build_envroot>", build_envroot) \
                     .replace("<mkdir>", mkdircmd) \
                     .replace("<runtime_envroot_mkdir>", runtime_envroot_mkdir) \
                     .replace("<copy>", copycmd) \
                     .replace("<runtime_envroot_cp>", runtime_envroot_cp)
-                cmdlist = [x for x in cmd.split() if x]
-                if DEBUG:
-                    print("COMMAND", cmdlist)
-                    p = subprocess.Popen(cmdlist)
-                else:
-                    p = subprocess.Popen(cmdlist, stdout=devnull, stderr=devnull)
-                p.wait()
-                time.sleep(10)
+                compiled_instructions.append(cmd)
+            instruction_file = fs_locations["build_exec"] + '/cptree_instructions.sh'
+            with open(instruction_file, 'w') as istf:
+                for ci in compiled_instructions:
+                    istf.write(ci+"\n")
+
+            cmdlist = ["bash", instruction_file]
+            if DEBUG:
+                print("COMMAND", " ".join(cmdlist))
+                p = subprocess.Popen(cmdlist)
+            else:
+                p = subprocess.Popen(cmdlist, stdout=devnull, stderr=devnull)
+            p.wait()
 
         if not workdir:
             print(("ERROR (generate_filesystem): Please specify the directory "
@@ -204,6 +212,8 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                 "filesystem specifications file, add #WORKDIR <path> where "
                 "<path> is a relative path from the environment root dir"))
             exit(1)
+        else:
+            workdir = workpath + '/' + workdir
 
         # Task files are in the Work sub-filesystem but outside the imported environment
         #  A cache in the main Work folder is created for containing them
@@ -238,6 +248,7 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
             subprocess.call(["chmod", "777", build_task_path])
 
         # This new cache has to be copied remotely, in a separate way
+        """
         if protocol == 'remote':
             tr_cmd = ["bash", data_transfer_protocol, build_cache, "{0}:{1}".format(remote_machine, runtime_cache)]
             if DEBUG:
@@ -247,7 +258,7 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                 p = subprocess.Popen(tr_cmd,
                     stderr=devnull, stdout=devnull)
             p.wait()
-            
+        """
 
     else:  # "locusts" mode: optimized environment for parallel and safe execution
     
@@ -284,7 +295,8 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
             rem_jf = (fs_locations["runtime_work"] + 'batch_'
                     + str(batchno) + '/' + 'task_' + jd['unique_code'] + '/')
             exec_filesystem[jd['unique_code']][0] = (rem_jf, "task.sh")
-    
+
+        """
         if protocol == "remote":
             # Transfer local temporary filesystem in the remote location
             tr_cmd = ["bash", data_transfer_protocol, fs_locations["build_work"], "{0}:{1}".format(remote_machine, fs_locations["runtime_work"])]
@@ -295,6 +307,7 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
                 p = subprocess.Popen(tr_cmd,
                     stderr=devnull, stdout=devnull)
             p.wait()
+        """
 
     return {k : exec_filesystem[k][0] for k in exec_filesystem}, fs_locations
 
@@ -344,8 +357,10 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
     # Is there singularity? Does it use a module?
     if singinfo[0]:
         singularitypath, singularitycont, singmodload = singinfo
+        singularitycmd = "{0} exec {1} ".format(singularitypath, singularitycont)
     else:
         singularitypath, singularitycont, singmodload = "", "", ""
+        singularitycmd = ""
 
     # List of task ids
     taskid_list = sorted([k for k in task_folders])
@@ -417,7 +432,7 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
                 .replace('<main_path>', fs_locations["runtime_root"]) \
                 .replace('<extra_outer_statements>', outer_statements) \
                 .replace('<singularity_module_load>', singmodload) \
-                .replace('<singularity_command>', "{0} exec {1} ".format(singularitypath, singularitycont)) \
+                .replace('<singularity_command>', singularitycmd) \
                 .replace('<inner_manager>', "inner_manager_{0}{1}.slurm".format(batch_job_code, str(jobid).zfill(3)))
 
             # Write manager file and give it exe privilege
@@ -443,7 +458,17 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
         # Job manager identifier and associated tasks
         tasks_per_job.append((jobid, taskid_list[ik:ik+tasks_per_node]))
 
-    
+    if protocol == 'remote':
+        cpall_cmd = ["bash", data_transfer_protocol, fs_locations["build_root"], "{0}:{1}".format(remote_machine, fs_locations["runtime_root"])]
+        if DEBUG:
+            print("COMMAND", " ".join(cpall_cmd))
+            p = subprocess.Popen(cpall_cmd)
+        else:
+            p = subprocess.Popen(cpall_cmd,
+                stderr=devnull, stdout=devnull)
+        out, err = p.communicate()
+
+    """ 
     if protocol == 'remote':
         # Copy all files in exe dir in the remote counterpart
         filestocopy = fs_locations["build_exec"]+"/"
@@ -455,7 +480,7 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
             p = subprocess.Popen(tr_cmd,
                 stderr=devnull, stdout=devnull)
         p.wait()
-
+    """
         # Remove the local root directory
 #        p = subprocess.Popen(
 #            ["rm", "-rf", fs_locations["build_root"]],
@@ -543,13 +568,13 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
             )
 
             if protocol == 'remote':
-                manager_cmd = ["ssh", remote_machine, "ls", "-latrh", macname]
+                manager_cmd = ["ssh", remote_machine, "stat", macname]
                 chk_time_cmd = "ssh " + remote_machine + " echo $(date +%H:%M)"
             elif protocol == 'remote-sharedfs':
-                manager_cmd = ["ls", "-latrh", macname_sf]
+                manager_cmd = ["stat", macname_sf]
                 chk_time_cmd = "echo $(date +%H:%M)"
             elif protocol == 'local':
-                manager_cmd = ["ls", "-latrh", macname]
+                manager_cmd = ["stat", macname]
                 chk_time_cmd = "echo $(date +%H:%M)"
             if DEBUG:
                 print("COMMAND", manager_cmd)
@@ -578,8 +603,16 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                         stdout=subprocess.PIPE, 
                         shell=True
                     ).stdout.read().decode('ascii').split()[0].split(":")
-                touch_time_l = touch_time_txt.split()[7].split(":")
-                touch_time = int(touch_time_l[0])*60 + int(touch_time_l[1])
+                touch_time = -9999
+                for line in touch_time_txt.split('\n'):
+                    fields = line.split()
+                    if fields[0] == "Modify:":
+                        ff = fields[2].split(":")
+                        touch_time = int(ff[0])*60 + int(ff[1])
+                        break
+                if touch_time == -9999:
+                    print("ERROR reading {0}\n{1}".format(manager_cmd, touch_time_txt))
+                    exit(1)
                 local_time = int(local_time_l[0])*60 + int(local_time_l[1])
                 coeff = (local_time - touch_time)%(24*60)
                 is_active = False if coeff > 1 else True
@@ -769,8 +802,12 @@ def gather_results(protocol_triad, cache_dir, job_data, batch_job_code,
             for k in job_data if k['unique_code'] in reschedule }
     
     elif snapshot:
-        new_snapshot = take_snapshot(protocol_triad, build_envroot)
+#        print("TAKE SNAPSHOT", protocol_triad, fs_locations['build_work'])
+        new_snapshot = take_snapshot(protocol_triad, fs_locations['build_work'])
+#        print(new_snapshot)
+#        print("NEWLY ADDED")
         newly_added = compare_snapshots(snapshot, new_snapshot)
+#        print(newly_added)
         snap_log = log_dir + 'modified.log'
         with open(snap_log, 'w') as snapf:
             # Creates all new folders
@@ -784,12 +821,13 @@ def gather_results(protocol_triad, cache_dir, job_data, batch_job_code,
             # Populates new and old folders
             for d in sorted(newly_added):
                 for f in sorted(newly_added[d]):
-                    snapf.write("File created/modified: {0}\n".format(build_envroot+f))
+                    snapf.write("File created/modified: {0}\n".format(build_envroot+d+'/'+f))
                     if protocol != 'local':
-                        fname = fs_locations["build_root"] + f  # Local copies have been restored at the beginning of this function
-                        cpcmd = ["cp", fname, build_envroot+f]
+                        fname = fs_locations["build_work"] + d+'/'+f  # Local copies have been restored at the beginning of this function
+                        cpcmd = ["cp", fname, build_envroot+d+'/'+f]
+#                        print("COMMAND", cpcmd)
                         if DEBUG:
-                            print("COMMAND", cpcmd)
+#                            print("COMMAND", cpcmd)
                             p = subprocess.Popen(cpcmd)
                         else:
                             p = subprocess.Popen(cpcmd, stdout=devnull, stderr=devnull)
@@ -797,7 +835,6 @@ def gather_results(protocol_triad, cache_dir, job_data, batch_job_code,
 
         output_d, output_paths, completed_with_error = {}, None, None
 
-
     # Remove all repositories
     if not DEBUG:
         if protocol != 'local':
@@ -833,8 +870,8 @@ def take_snapshot(protocol_triad, root_dir):
     """
 
     lsrcmd = ["ls", "-ltrhR", root_dir]
-    if remote_machine:
-        lsrcmd = ["ssh", remote_machine] + lsrcmd
+#    if remote_machine:
+#        lsrcmd = ["ssh", remote_machine] + lsrcmd
 
     if DEBUG:
         print("COMMAND", lsrcmd)
@@ -850,10 +887,11 @@ def take_snapshot(protocol_triad, root_dir):
         if not line.strip() or line.startswith('total'):
             continue
         if len(line.split()) == 1:
-            dirname = line.strip()[:-1]  # line terminates with ":"
+            dirname = line.strip().replace(root_dir,"")[:-1]  # line terminates with ":"
             snapshot[dirname] = {}
         elif len(line.split()) == 9:
-            priv, _, usr1, usr2, s, d1, d2, d3, filename = line.split() 
+            priv, _, usr1, usr2, s, d1, d2, d3, filename = line.split()
+            filename = filename.replace(root_dir,"")
             if priv.startswith('-'):  # If item is not a folder
                 snapshot[dirname][filename] = (priv, usr1, usr2, s, d1, d2, d3)
         else:
@@ -1033,6 +1071,9 @@ def highly_parallel_job_manager(options, exec_filename,
                 (protocol, "", local_shared_folder), 
                 env_root_dir
             )
+
+#            print("FIRST SNAPSHOT", (protocol, "", local_shared_folder), env_root_dir)
+#            print(snapshot)
             if not snapshot:
                 print("ERROR: empty snapshot of {0} was taken".format(env_root_dir))
                 print("    Locusts needs to take snapshot to compare filesystems")
